#include "Game.h"
#include "Player.h"
#include "Bomber.h"
#include "GameObject.h"
#include "Rectangle.h"
#include "InputHandler.h"
#include "FlyingNum.h"
#include "Bucket.h"
#include "Bomb.h"
#include "BombHandler.h"
#include "Collision.h"

Game* Game::s_pInstance = 0;

Game::Game(){}

void Game::update()
{
  // loop through and update our objects
  for(std::vector<GameObject*>::size_type i = 0; i !=   m_gameObjects.size(); i++)
  {
    m_gameObjects[i]->update();
  }
  for(std::vector<Bucket*>::size_type i = 0; i !=   m_buckets.size(); i++)
  {
    m_buckets[i]->update();
  }
  m_bomber->update();

  // check for collisions
  std::vector<Bomb*> bombs = BombHandler::Instance()->getBombs();
  //m_buckets
  // use iterator like in bombhandler

  for (std::vector<Bomb*>::iterator p_it = bombs.begin(); p_it != bombs.end();) {
	  bool collide = false;
      SDL_Rect* pRect1 = new SDL_Rect();
	pRect1->x = (*p_it)->getPosition().getX();
	pRect1->y = (*p_it)->getPosition().getY();
	pRect1->w = (*p_it)->getWidth();
	pRect1->h = (*p_it)->getHeight();

	for(std::vector<Bucket *>::iterator b_it = m_buckets.begin() ; b_it != m_buckets.end() ;)
	{
	      SDL_Rect* pRect2 = new SDL_Rect();
		pRect2->x = (*b_it)->getPosition().getX();
		pRect2->y = (*b_it)->getPosition().getY();
		pRect2->w = (*b_it)->getWidth();
		pRect2->h = (*b_it)->getHeight();
		if (RectRect(pRect1,pRect2)) {
			collide = true;
			(*p_it)->setCaught();
		}
		++b_it;
		delete(pRect2);
	}

	  ++p_it;
      delete( pRect1);
  }


}

bool Game::init(const char* title, int xpos, int ypos, int width, int height, bool fullscreen)
{

	  m_gameWidth = width;
	  m_gameHeight = height;

  int flags = 0;

  if(fullscreen)
  {
    flags = SDL_WINDOW_FULLSCREEN;
  }

  // attempt to initialize SDL
  if(SDL_Init(SDL_INIT_EVERYTHING) == 0)
  {
    std::cout << "SDL init success\n";
    // init the window
    m_pWindow = SDL_CreateWindow(title, xpos, ypos, 
    width, height, flags);

    if(m_pWindow != 0) // window init success
    {
      std::cout << "window creation success\n";
      m_pRenderer = SDL_CreateRenderer(m_pWindow, -1, 0);

      if(m_pRenderer != 0) // renderer init success
      {
        std::cout << "renderer creation success\n";
        SDL_SetRenderDrawColor(m_pRenderer, 0,0,0,255);

	//SDL_Surface* pTempSurface = SDL_LoadBMP("./assets/rider.bmp");
		//SDL_Surface* pTempSurface = SDL_LoadBMP("./assets/animate.bmp");
		//SDL_Surface* pTempSurface = IMG_Load("./assets/animate-alpha.png");

		//m_textureManager.load("assets/animate-alpha.png", "animate", m_pRenderer);



	if(!TheTextureManager::Instance()->load(DATADIR"/background.png", "background", m_pRenderer))
	{
	   return false;
	}

	if(!TheTextureManager::Instance()->load(DATADIR"/bomber-happy.png", "bomber-happy", m_pRenderer))
	{
	   return false;
	}

	if(!TheTextureManager::Instance()->load(DATADIR"/bucket_sheet.png", "bucket", m_pRenderer))
	{
	   return false;
	}

	if(!TheTextureManager::Instance()->load(DATADIR"/bomb_sheet.png", "bomb", m_pRenderer))
	{
	   return false;
	}


	//m_background = new Background();
	m_background = new GameObject();
	m_bomber = new Bomber();
	m_rect = new Rectangle();
	fly_num = new FlyingNum(1);
	m_bucket1 = new Bucket(0);
	m_bucket2 = new Bucket(1);
	m_bucket3 = new Bucket(2);
	m_bomb = new Bomb();


	m_rect->load(50,50,100,20);


	m_bucket1->load(100, 400, 64, 32, "bucket");
	m_bucket2->load(100, 360, 64, 32, "bucket");
	m_bucket3->load(100, 320, 64, 32, "bucket");

	m_background->load(0, 0, 640, 480, "background");

	m_bomber->load(30, 65, 32, 64, "bomber-happy");
	m_bomb->load(30,65,32,32, "bomb");
	m_bomb->setFall(1);

	m_gameObjects.push_back(m_background);




	//m_gameObjects.push_back(m_bucket1);
	//m_gameObjects.push_back(m_bucket2);
	//m_gameObjects.push_back(m_bucket3);

	m_buckets.push_back(m_bucket1);
	m_buckets.push_back(m_bucket2);
	m_buckets.push_back(m_bucket3);
	//m_gameObjects.push_back(m_bomb);

	//m_gameObjects.push_back(m_bomber);
	//m_gameObjects.push_back(m_rect);

	m_gameObjects.push_back(fly_num);


	//m_gameObjects.push_back(m_go);


      }
      else
      {
        std::cout << "renderer init fail\n";
        return false; // renderer init fail
      }
    }
    else
    {
      std::cout << "window init fail\n";
      return false; // window init fail
    }
  }
  else
  {
    std::cout << "SDL init fail\n";
    return false; // SDL init fail
  }

  std::cout << "init success\n";
  m_bRunning = true; // everything inited successfully, start the main loop

 



  return true;
}

void Game::render()
{
  SDL_RenderClear(m_pRenderer); // clear to the draw colour

  // loop through our objects and draw them
  for(std::vector<GameObject*>::size_type i = 0; i != 
  m_gameObjects.size(); i++)
  {
    m_gameObjects[i]->draw(m_pRenderer);
  }

  // render buckets
  for(std::vector<GameObject*>::size_type i = 0; i !=
  m_buckets.size(); i++)
  {
    m_buckets[i]->draw(m_pRenderer);
  }

  m_bomber->draw(m_pRenderer);

  SDL_RenderPresent(m_pRenderer); // draw to the screen
}

void Game::clean()
{
  std::cout << "cleaning game\n";
  SDL_DestroyWindow(m_pWindow);
  SDL_DestroyRenderer(m_pRenderer);
  SDL_Quit();
}

void Game::handleEvents()
{
	TheInputHandler::Instance()->update();
	if (TheInputHandler::Instance()->quit())
		m_bRunning = false;

}

void Game::draw()
{
  for(std::vector<GameObject*>::size_type i = 0; i !=   m_gameObjects.size(); i++) 
  {
    m_gameObjects[i]->draw(m_pRenderer);
  }
}
